/****************************************************************
   @PRG_NAME: AT89C51xx-Flash
   @PRG_VERS: V00.01.00
   @PRG_AUTH: Rene Schube
   @PRG_LICS: GNU/GPL V3.x
   @PRG_DATE: 2011

   @FILE_NAME: main.c
 ****************************************************************/
#include "main.h"

char  cError;

/****************************************************************
 ****************************************************************/
int main (int argc, char *argv[])
{
   int            iRetVal;
   unsigned int   uiStart;
   unsigned int   uiEnde;
   unsigned int   uiRange;
   char           cBuffer;
   char           cData[16];
   char           c;
   FILE           *fp;
   

   if (InitCVIRTE (0, argv, 0) == 0)
      return -1;    /* out of memory */
   
   // Setzte ErrorFlag auf 0x00
   cError = 0x00;
   
   // Start Console
   printf("%s\n", PRG_NAME);

   // COM1 open
   iRetVal = OpenComPort();
   
   // HandShake
   // PC -->  'U'
   // uC -->  'U' - PASS
   // uc --> !'U' - FAIL
   iRetVal = Write_Byte ( 'U' );
   if ( (cBuffer = Read_Byte ()) != 'U' )
   {
      printf("Not Received \"U\" [ ASCII = '%c' hex = '0x%.X' dec = '%d' ]\n", cBuffer, cBuffer, cBuffer);
      cError = 0x01;
   }
   
   /*-- Read Functions -------------------------------------------*/
   cData[0] = 0x00;
   cData[1] = 0x00;
   iRetVal = CalcRecord(0x02, 0x0000, 0x05, cData);
   printf("Record 01 [%s] Read Manufacture Id\n", cBufArray);
   iRetVal = Write_Record ( cBufArray );
   while ( (cBuffer = Read_Byte ()) != 10 )
      printf("%c", cBuffer);
   printf("\n");

   cData[0] = 0x00;
   cData[1] = 0x01;
   iRetVal = CalcRecord(0x02, 0x0000, 0x05, cData);
   printf("Record 02 [%s] Read Device Id #1\n", cBufArray);
   iRetVal = Write_Record ( cBufArray );
   while ( (cBuffer = Read_Byte ()) != 10 )
      printf("%c", cBuffer);
   printf("\n");

   
   cData[0] = 0x00;
   cData[1] = 0x02;
   iRetVal = CalcRecord(0x02, 0x0000, 0x05, cData);
   printf("Record 03 [%s] Read Device Id #2\n", cBufArray);
   iRetVal = Write_Record ( cBufArray );
   while ( (cBuffer = Read_Byte ()) != 10 )
      printf("%c", cBuffer);
   printf("\n");
   
   cData[0] = 0x00;
   cData[1] = 0x03;
   iRetVal = CalcRecord(0x02, 0x0000, 0x05, cData);
   printf("Record 04 [%s] Read Device Id #3\n", cBufArray);
   iRetVal = Write_Record ( cBufArray );
   while ( (cBuffer = Read_Byte ()) != 10 )
      printf("%c", cBuffer);
   printf("\n");
   
   cData[0] = 0x07;
   cData[1] = 0x00;
   iRetVal = CalcRecord(0x02, 0x0000, 0x05, cData);
   printf("Record 05 [%s] Read Software Security Byte SSB\n", cBufArray);
   iRetVal = Write_Record ( cBufArray );
   while ( (cBuffer = Read_Byte ()) != 10 )
      printf("%c", cBuffer);
   printf("\n");
   
   cData[0] = 0x07;
   cData[1] = 0x02;
   iRetVal = CalcRecord(0x02, 0x0000, 0x05, cData);
   printf("Record 06 [%s] Read Software Boot Vectro SBV\n", cBufArray);
   iRetVal = Write_Record ( cBufArray );
   while ( (cBuffer = Read_Byte ()) != 10 )
      printf("%c", cBuffer);
   printf("\n");
   
   cData[0] = 0x07;
   cData[1] = 0x06;
   iRetVal = CalcRecord(0x02, 0x0000, 0x05, cData);
   printf("Record 07 [%s] Read Extra Byte\n", cBufArray);
   iRetVal = Write_Record ( cBufArray );
   while ( (cBuffer = Read_Byte ()) != 10 )
      printf("%c", cBuffer);
   printf("\n");
   
   cData[0] = 0x0B;
   cData[1] = 0x00;
   iRetVal = CalcRecord(0x02, 0x0000, 0x05, cData);
   printf("Record 08 [%s] Read Hardware Byte\n", cBufArray);
   iRetVal = Write_Record ( cBufArray );
   while ( (cBuffer = Read_Byte ()) != 10 )
      printf("%c", cBuffer);
   printf("\n");
   
   cData[0] = 0x0E;
   cData[1] = 0x00;
   iRetVal = CalcRecord(0x02, 0x0000, 0x05, cData);
   printf("Record 09 [%s] Read Device Boot Id #1\n", cBufArray);
   iRetVal = Write_Record ( cBufArray );
   while ( (cBuffer = Read_Byte ()) != 10 )
      printf("%c", cBuffer);
   printf("\n");
   
   cData[0] = 0x0E;
   cData[1] = 0x01;
   iRetVal = CalcRecord(0x02, 0x0000, 0x05, cData);
   printf("Record 10 [%s] Read Device Boot Id #2\n", cBufArray);
   iRetVal = Write_Record ( cBufArray );
   while ( (cBuffer = Read_Byte ()) != 10 )
      printf("%c", cBuffer);
   printf("\n");
   
   cData[0] = 0x0F;
   cData[1] = 0x00;
   iRetVal = CalcRecord(0x02, 0x0000, 0x05, cData);
   printf("Record 11 [%s] Read Bootloader Version\n", cBufArray);
   iRetVal = Write_Record ( cBufArray );
   while ( (cBuffer = Read_Byte ()) != 10 )
      printf("%c", cBuffer);
   printf("\n");
   
   /*-- Write Functions -------------------------------------------*/
   cData[0] = 0x01;
   cData[1] = 0x00;
   iRetVal = CalcRecord(0x02, 0x0000, 0x03, cData);
   uiStart = 0x0000;
   uiEnde  = 0x1FFF;
   uiRange = uiEnde - uiStart;
   printf("Record 12 [%s] Erase Block 0 0x%.4X-0x%.4X  %.4d Byte  %dk\n", 
         cBufArray, uiStart, uiEnde, uiRange, (uiRange / 1023) );
   iRetVal = Write_Record ( cBufArray );
   while ( (cBuffer = Read_Byte ()) != 10 )
      printf("%c", cBuffer);
   printf("\n");
   
   cData[0] = 0x01;
   cData[1] = 0x20;
   iRetVal = CalcRecord(0x02, 0x0000, 0x03, cData);
   uiStart = 0x2000;
   uiEnde  = 0x3FFF;
   uiRange = uiEnde - uiStart;
   printf("Record 13 [%s] Erase Block 1 0x%.4X-0x%.4X  %.4d Byte  %dk\n", 
         cBufArray, uiStart, uiEnde, uiRange, (uiRange / 1023) );
   iRetVal = Write_Record ( cBufArray );
   while ( (cBuffer = Read_Byte ()) != 10 )
      printf("%c", cBuffer);
   printf("\n");
   
   cData[0] = 0x01;
   cData[1] = 0x40;
   iRetVal = CalcRecord(0x02, 0x0000, 0x03, cData);
   uiStart = 0x4000;
   uiEnde  = 0x7FFF;
   uiRange = uiEnde - uiStart;
   printf("Record 14 [%s] Erase Block 2 0x%.4X-0x%.4X  %.4d Byte  %dk\n", 
         cBufArray, uiStart, uiEnde, uiRange, (uiRange / 1023) );
   iRetVal = Write_Record ( cBufArray );
   while ( (cBuffer = Read_Byte ()) != 10 )
      printf("%c", cBuffer);
   printf("\n");
   
   cData[0] = 0x01;
   cData[1] = 0x80;
   iRetVal = CalcRecord(0x02, 0x0000, 0x03, cData);
   uiStart = 0x8000;
   uiEnde  = 0xBFFF;
   uiRange = uiEnde - uiStart;
   printf("Record 15 [%s] Erase Block 3 0x%.4X-0x%.4X  %.4d Byte  %dk\n", 
         cBufArray, uiStart, uiEnde, uiRange, (uiRange / 1023) );
   iRetVal = Write_Record ( cBufArray );
   while ( (cBuffer = Read_Byte ()) != 10 )
      printf("%c", cBuffer);
   printf("\n");
   
   cData[0] = 0x01;
   cData[1] = 0xC0;
   iRetVal = CalcRecord(0x02, 0x0000, 0x03, cData);
   uiStart = 0xC000;
   uiEnde  = 0xFFFF;
   uiRange = uiEnde - uiStart;
   printf("Record 16 [%s] Erase Block 4 0x%.4X-0x%.4X  %.4d Byte  %dk\n", 
         cBufArray, uiStart, uiEnde, uiRange, (uiRange / 1023) );
   iRetVal = Write_Record ( cBufArray );
   while ( (cBuffer = Read_Byte ()) != 10 )
      printf("%c", cBuffer);
   printf("\n");
   
   // BLANKCHECK (over 0x0000 - 0x7FFF)
   cData[0] = 0x00;
   cData[1] = 0x00;
   cData[2] = 0x7F;
   cData[3] = 0xFF;
   cData[4] = 0x01;
   iRetVal = CalcRecord(0x05, 0x0000, 0x04, cData);
   uiStart = 0x0000;
   uiEnde  = 0x7FFF;
   uiRange = uiEnde - uiStart;
   printf("Record 17 [%s] Blank Check 0x%.4X-0x%.4X  %.4d Byte  %dk\n", 
         cBufArray, uiStart, uiEnde, uiRange, (uiRange / 1023) );
   iRetVal = Write_Record ( cBufArray );
   while ( (cBuffer = Read_Byte ()) != 10 )
      printf("%c", cBuffer);
   //if ( cBuffer == '.' )
   //   printf("Blank Check PASS\n");
   //else if ( cBuffer == 'X' )
   //   printf("Blank Check FAIL with checksum error\n");


   // PROGRAMMIERUNG
   printf("Programmierung:\n");
   sprintf(cBufArray,"\0");

   // HEX-File laden
   fp = fopen("MWE200.hex", "r");
   if ( fp != NULL )
   {
      while (( c = fgetc(fp)) != EOF)
      {
         if ( c != 10 ) {     // dec = 10; 0x0D; ASCII '\n'
            sprintf( cBufArray, "%s%c", cBufArray, c );
         } else {
            iRetVal = Write_Record ( cBufArray );
            while ( (cBuffer = Read_Byte ()) != 10 )
               printf("%c", cBuffer);
            sprintf( cBufArray, "\0");
            printf("\n");
         }
      }
   } else {
      printf("Konnte Datei nicht oeffnen\n");
   }

   // HARDWARE RESET
   cData[0] = 0x03;
   cData[1] = 0x00;
   iRetVal = CalcRecord(0x02, 0x0000, 0x03, cData);
   printf("Record 18 [%s] Hardware Reset\n", cBufArray);


/*
   cData[0] = 0x04;
   cData[1] = 0x00;
   iRetVal = CalcRecord(0x02, 0x0000, 0x03, cData);
   printf("Record 18 [%s] Erease SBV & BSB\n", cBufArray);
   
   cData[0] = 0x05;
   cData[1] = 0x00;
   iRetVal = CalcRecord(0x02, 0x0000, 0x03, cData);
   printf("Record 19 [%s] Program SSB Level 1\n", cBufArray);
   
   cData[0] = 0x05;
   cData[1] = 0x01; 
   iRetVal = CalcRecord(0x02, 0x0000, 0x03, cData);
   printf("Record 20 [%s] Program SSB Level 2\n", cBufArray);
   
   cData[0] = 0x06;
   cData[1] = 0x00;
   cData[2] = 0xAA; // ACHTUNG Byte aus Datenblatt 
   iRetVal = CalcRecord(0x03, 0x0000, 0x03, cData);
   printf("Record 21 [%s] Program BSB (value = cData[2])\n", cBufArray);
   
   cData[0] = 0x06;
   cData[1] = 0x01;
   cData[2] = 0x55; // ACHTUNG Byte aus Datenblatt 
   iRetVal = CalcRecord(0x03, 0x0000, 0x03, cData);
   printf("Record 22 [%s] Program SBV (value = cData[2])\n", cBufArray);
   
   cData[0] = 0x07;
   iRetVal = CalcRecord(0x01, 0x0000, 0x03, cData);
   printf("Record 23 [%s] Full Chip Erase (about 6 sec to be executed)\n", cBufArray);   
   
   cData[0] = 0x0A;
   cData[1] = 0x04;
   cData[2] = 0x55; // ACHTUNG Byte aus Datenblatt 
   iRetVal = CalcRecord(0x03, 0x0000, 0x03, cData);
   printf("Record 24 [%s] Program BLJB (value = cData[2])\n", cBufArray);
   
   cData[0] = 0x0A;
   cData[1] = 0x08;
   cData[2] = 0xAA; // ACHTUNG Byte aus Datenblatt 
   iRetVal = CalcRecord(0x03, 0x0000, 0x03, cData);
   printf("Record 25 [%s] Program X2 fuse (value = cData[2])\n", cBufArray);
*/ 
   // Schliesse COM-Port
   iRetVal = CloseComPort();

   // Ende Console
   printf("\n\nProgram Name: %s %s\n", PRG_NAME, PRG_VERS);
   printf("%s CopyLeft %s by %s\n", PRG_LICS, PRG_DATE, PRG_AUTH);
   printf("Weiter mit Return-Taste.\n");
   // Programm ende
   getchar ();
   return 0;
}

// EOF - End of File
// Do not type any code after this line
