/****************************************************************
   @PRG_NAME: AT89C51xx-Flash
   @PRG_VERS: V00.01.00
   @PRG_AUTH: Rene Schube
   @PRG_LICS: GNU/GPL V3.x
   @PRG_DATE: 2011

   @FILE_NAME: rs232.c
 ****************************************************************/
#include <ansi_c.h>
#include <utility.h>
#include <rs232.h>

#include "main.h"

// FUNCTION DECLARATION
int OpenComPort ( void );
int Write_Byte ( char cByte );
int Write_Record ( char cBufArray[128+1] );
char Read_Byte ( void );
int CloseComPort ( void );

// FUNCTION

/****************************************************************
 ****************************************************************/
int OpenComPort ( void )
{
   int   iRetVal;
   int   iStatus;
   
   iRetVal = OpenComConfig (1, "COM1", 38400, 0, 8, 1, 512, 512);
   
   iStatus = ReturnRS232Err ();
   if ( iStatus != 0 )
   {
      printf("Fehler %d - %s\n", iStatus, GetRS232ErrorString (iStatus));
      return -1;
   }
   return 0;
}

/****************************************************************
 ****************************************************************/
int Write_Byte ( char cByte )
{
   int   iStatus;
   
   ComWrtByte (1, cByte);
   
   iStatus = ReturnRS232Err ();
   if ( iStatus != 0 )
   {
      printf("Fehler %d - %s\n", iStatus, GetRS232ErrorString (iStatus));
      return -1;
   }
   return 0;
}

/****************************************************************
 ****************************************************************/
int Write_Record ( char cBufArray[128+1] )
{
   int   iStatus;
   int   iBytesWritten;
   
   iBytesWritten = ComWrt (1, cBufArray, strlen(cBufArray));
   
   iStatus = ReturnRS232Err ();
   if ( iStatus != 0 )
   {
      printf("Fehler %d - %s\n", iStatus, GetRS232ErrorString (iStatus));
      return -1;
   }
   return 0;
}

/****************************************************************
 ****************************************************************/
char Read_Byte ( void )
{
   char  cByte;
   int   iError;
   char  cBuffer;
   
   Delay (0.010);
   ComRd (1, &cByte, 1);
   
   iError = ReturnRS232Err ();
   if ( iError < 0 ) {
      printf("Fehler %d - %s\n", iError, GetRS232ErrorString (iError));
      return -1;
   }
   
   return cByte;
}

/****************************************************************
 ****************************************************************/
int CloseComPort ( void )
{
   int   iStatus;
   
   CloseCom (1);
   
   iStatus = ReturnRS232Err ();
   if ( iStatus != 0 )
   {
      printf("Fehler %d - %s\n", iStatus, GetRS232ErrorString (iStatus));
      return -1;
   }
   return 0;
}

// EOF - End of File
// Do not type any code after this line
